@@ -10,13 +10,13 @@ import socket
 import sqlite3
 import winreg
 import xml.etree.ElementTree as ElTree
+import zipfile
 import zlib
 from datetime import datetime, timedelta
 from inspect import signature
 from shutil import copyfile
 from typing import List, Any
 from urllib import parse
-
 import xmltodict
 
 
@@ -104,11 +104,14 @@ class WorkflowEngine:
         self.step_nr = None
         self.variables = {}  # Dictionary to hold WorkflowEngine variables
 
-    def get_input_parameter(self):
+    def get_input_parameter(self, as_dictionary: False):
         """
         Returns the input parameter that was given when creating an instance of the WorkflowEngine
+        :param as_dictionary: Optional. Indicator whether to treat the given input as a dictionary object (string to dict).
         :return: The input_parameter that was given when creating an instance of the WorkflowEngine
         """
+        if as_dictionary:
+            self.input_parameter = eval(self.input_parameter)
         self.print_log(f"Got input parameter {str(self.input_parameter)}")
         return self.input_parameter
 
@@ -123,16 +126,23 @@ class WorkflowEngine:
         if filepath is not None:
             self.flowpath = filepath
         xml_file = open(filepath, "r")
-        self.flowname = filepath.split("\\")[-1].lower().replace(".xml", "")
-        xml_root = ElTree.fromstring(xml_file.read())
-        raw_text = xml_root[0].text
-        base64_decode = base64.b64decode(raw_text)
-        inflated_xml = zlib.decompress(base64_decode, -zlib.MAX_WBITS).decode("utf-8")
-        url_decode = parse.unquote(inflated_xml)
-        if as_xml:
-            return url_decode
-        retn = xmltodict.parse(url_decode)
-        return retn
+        if not filepath.__contains__(".vsdx"):
+            self.flowname = filepath.split("\\")[-1].lower().replace(".xml", "")
+            xml_root = ElTree.fromstring(xml_file.read())
+            raw_text = xml_root[0].text
+            base64_decode = base64.b64decode(raw_text)
+            inflated_xml = zlib.decompress(base64_decode, -zlib.MAX_WBITS).decode("utf-8")
+            url_decode = parse.unquote(inflated_xml)
+            if as_xml:
+                return url_decode
+            retn = xmltodict.parse(url_decode)
+            return retn
+        else:
+            # It is a MsVisio file!
+            visio = Visio()
+            self.flowname = filepath.split("\\")[-1].lower().replace(".vsdx", "")
+            visio.open_vsdx_file(filepath)
+            return visio
 
     @staticmethod
     def set_db_path(value: str):
@@ -198,12 +208,16 @@ class WorkflowEngine:
         except WindowsError:
             return False
 
-    def get_flow(self, ordered_dict: dict) -> Any:
+    def get_flow(self, ordered_dict: Any) -> Any:
         """
         Retrieving the elements of the flow in the Document.
         :param ordered_dict: The document object containing the flow elements.
         :returns: A List of flow elements
         """
+        if str(ordered_dict).__contains__("Visio object"):
+            # It is a Visio Object!
+            visio = ordered_dict
+            return visio.get_flow()
         connectors = []
         shapes = []
         connectorvalues = {}
@@ -339,7 +353,10 @@ class WorkflowEngine:
                 if val is not None:
                     if len(str(val)) == 0:
                         if input is not None:
-                            val = ""
+                            if str(value.default) == "None":
+                                val = None
+                            else:
+                                val = ""
                         else:
                             if str(value).__contains__("="):
                                 val = value.default
@@ -561,7 +578,8 @@ class WorkflowEngine:
                 self.flowpath):
             # Move the file to the registered directory if not exists
             copyfile(self.flowpath, f'{db_path}\\Registered Flows\\{self.flowname}.xml')
-        self.flowpath = f'{db_path}\\Registered Flows\\{self.flowname}.xml'
+        if not self.flowpath.lower().__contains__(".vsdx"):
+            self.flowpath = f'{db_path}\\Registered Flows\\{self.flowname}.xml'
         sql = f"SELECT id FROM Flows WHERE name ='{self.flowname}' AND location='{self.flowpath}'"
         flow_id = self.db.run_sql(sql=sql, tablename="Flows")
         if flow_id is None:
@@ -1139,7 +1157,7 @@ class SQL:
             ret.append(f"{rw[0]}.xml")
         return ret
 
-    def get_runned_flows(self, flow_id = None):
+    def get_runned_flows(self, flow_id=None):
         """
         Get a list of all runned flows in the orchestrator database.
         :param flow_id: Optional. The flow ID to get the runned data of.
@@ -1197,8 +1215,280 @@ class SQL:
         # sql = "CREATE TABLE IF NOT EXISTS Triggers (id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, flow_id INTEGER NOT NULL, trigger_info, CONSTRAINT flows_saved_trigger FOREIGN KEY (flow_id) REFERENCES Flows (id) ON DELETE CASCADE);"
         # self.run_sql(sql)
 
+
+class Visio:
+
+    def __init__(self):
+        """
+        Class for reading a flow in MsVisio format (vsdx).
+        """
+        self.root = None
+        self.master_connection = []
+        self.master_shape = []
+
+    class dynamic_object(object):
+        pass
+
+    def open_vsdx_file(self, file: str):
+        """
+        Open the VSDX file and store its cointents into memory.
+        :param file: The filename to read.
+        """
+        docs = zipfile.ZipFile(file, "r")
+        self.root = {}
+        for d in docs.filelist:
+            # print(d)
+            if str(d.filename).lower().endswith(".xml") or str(d.filename).lower().endswith(".rels"):
+                doc = docs.read(d)
+                dct = xmltodict.parse(doc)
+                self.root.update({d.filename: dct})
+
+    def get_start(self):
+        """
+        Return the start Shape of the flow.
+        """
+        count = 1
+        while True:
+            if f"visio/pages/page{count}.xml" in self.root:
+                for shape in self.root[f"visio/pages/page{count}.xml"].get("PageContents").get("Shapes").get("Shape"):
+                    if "Section" in shape:
+                        section = shape.get("Section")
+                        if not isinstance(section, list):
+                            section = [section]
+                        for sec in section:
+                            if sec["@N"] == "Property":
+                                if not isinstance(sec["Row"], list):
+                                    if {"@N": "Label", "@V": "Type"} in sec["Row"].get("Cell") and {"@N": "Value", "@V": "Start", "@U": "STR"} in sec["Row"].get("Cell"):
+                                        setattr(shape, "IsStart", True)
+                                        return shape
+                                else:
+                                    for rw in sec["Row"]:
+                                        if {"@N": "Label", "@V": "Type"} in rw.get("Cell") and {"@N": "Value", "@V": "Start", "@U": "STR"} in rw.get("Cell"):
+                                            setattr(shape, "IsStart", True)
+                                            return shape
+                count += 1
+            else:
+                break
+        return None
+
+    def get_connectors(self) -> list:
+        """
+        Get all connectors from the flow.
+        :return: A list of connectors.
+        """
+        retn = []
+        count = 1
+        while True:
+            if f"visio/pages/page{count}.xml" in self.root:
+                connects = self.root[f"visio/pages/page{count}.xml"].get("PageContents").get("Connects").get("Connect")
+                if connects:
+                    for shp in self.root[f"visio/pages/page{count}.xml"].get("PageContents").get("Connects").get("Connect"):
+                        conn = {"@ID": shp["@FromSheet"], "type": "connector"}
+                        if shp["@FromCell"] == "BeginX":
+                            obj = [x for x in self.root[f"visio/pages/page{count}.xml"].get("PageContents").get("Shapes").get("Shape") if x["@ID"] == shp["@FromSheet"]][0]
+                            text = None
+                            if "Text" in obj:
+                                text = obj["Text"]
+                            if text is not None:
+                                conn.update({"value": text})
+                            conn.update({"source": shp["@ToSheet"]})
+                            target = [y for y in [x for x in connects if x["@FromSheet"] == shp["@FromSheet"]] if y["@FromCell"] == "EndX"][0]["@ToSheet"]
+                            conn.update({"target": target})
+                            retn.append(conn)
+                    count += 1
+            else:
+                break
+        return retn
+
+    def check_dimensions(self, shape: Any) -> Any:
+        """
+        Get Width and Height attributes from master if not already in shape attributes.
+        :param shape: The shape object to check.
+        :return: The shape object including height and width attributes.
+        """
+        if [x for x in shape["Cell"] if x["@N"] == "Width"] and [x for x in shape["Cell"] if x["@N"] == "Height"]:
+            return shape
+        master = [x for x in self.root[f"visio/masters/masters.xml"].get("Masters").get("Master") if x["@ID"] == shape["@Master"]][0]
+        rel = master["Rel"].get("@r:id")
+        target = [x for x in self.root[f"visio/masters/_rels/masters.xml.rels"].get("Relationships").get("Relationship") if x["@Id"] == rel][0]["@Target"]
+        w = [x for x in self.root[f"visio/masters/{target}"].get("MasterContents").get("Shapes").get("Shape").get("Cell") if x["@N"] == "Width"][0]
+        h = [x for x in self.root[f"visio/masters/{target}"].get("MasterContents").get("Shapes").get("Shape").get("Cell") if x["@N"] == "Height"][0]
+        shape["Cell"].append(w)
+        shape["Cell"].append(h)
+        return shape
+
+    def get_flow(self):
+        """
+        Get the flow from the file.
+        :return: The flow object.
+        """
+        flow_ = []
+        shape = self.get_start()
+        shape = self.check_dimensions(shape)
+        flow_.append(shape)
+        count = 1
+        while True:
+            if f"visio/pages/page{count}.xml" in self.root:
+                shapes = self.root[f"visio/pages/page{count}.xml"].get("PageContents").get("Shapes").get("Shape")
+                for shp in shapes:
+                    if "@NameU" in shp:
+                        if str(shp["@NameU"]).lower().__contains__("connector"):
+                            if shp["@Master"] not in self.master_connection:
+                                self.master_connection.append(shp["@Master"])
+                    if not flow_.__contains__(shp) and not shp["@Master"] in self.master_connection:
+                        flow_.append(shp)
+                count += 1
+            else:
+                break
+        connectors = self.get_connectors()
+        flow_ += connectors
+        retn = []
+        for s in flow_:
+            step = self.get_step_from_shape(s)
+            retn.append(step)
+        return retn
+
+    def get_step_from_shape(self, shape: Any) -> Any:
+        """
+        Build a Step-object from the Shape-object
+        :param shape: The Shape-object
+        :returns: A Step-object
+        """
+        properties = {"id": shape.get("@ID")}
+        properties.update({"type": "shape"})
+        if "@Name" in shape:
+            if "Text" in shape:
+                properties.update({"name": shape["Text"]})
+            else:
+                properties.update({"name": shape["@Name"]})
+        properties.update({"IsStart": False})
+        retn = self.dynamic_object()
+        if "type" in shape:
+            if str(shape["type"]).lower().__contains__("connector"):
+                properties.pop("IsStart")
+                properties.update({"type": "connector"})
+                properties.update({"target": shape["target"]})
+                properties.update({"source": shape["source"]})
+                if "value" in shape:
+                    properties.update({"value": shape["value"]})
+                    properties.update({"name": shape["value"]})
+                for k, v in properties.items():
+                    setattr(retn, k, v)
+                return retn
+
+        # Get default attributes if any
+        standard_attributes = []
+        props = []
+        target_id = None
+        # region attributes from template
+        if "@Master" in shape:
+            master = shape["@Master"]
+            obj = [x for x in self.root[f"visio/masters/masters.xml"].get("Masters").get("Master") if x["@ID"] == master]
+            if obj:
+                r_id = obj[0]["Rel"]["@r:id"]
+                obj = [x for x in self.root[f"visio/masters/_rels/masters.xml.rels"].get("Relationships").get("Relationship") if x["@Id"] == r_id]
+                if obj:
+                    target_id = obj[0]["@Target"]
+                    obj = [x for x in self.root[f"visio/masters/{target_id}"]["MasterContents"]["Shapes"]["Shape"]["Section"] if x["@N"] == "Property"]
+                    if obj:
+                        if "Cell" in obj[0]["Row"]:
+                            standard_attributes = obj[0]["Row"]["Cell"]
+                        else:
+                            for rw in obj[0]["Row"]:
+                                for attr in rw["Cell"]:
+                                    standard_attributes.append(attr)
+        key = None
+        value = None
+        for rw in standard_attributes:
+            if rw["@N"] == "Label":
+                key = rw["@V"]
+            if rw["@N"] == "Value":
+                value = rw["@V"]
+            if key is not None and value is not None:
+                properties.update({"row_id": "template", str(key).lower(): value})
+                # reset key and value
+                key, value = None, None
+        labels = []
+        if target_id is not None:
+            obj = [x for x in self.root[f"visio/masters/{target_id}"]["MasterContents"]["Shapes"]["Shape"]["Section"] if x["@N"] == "Property"]
+            if obj:
+                labels = obj[0]["Row"]
+        # endregion
+        if "Section" in shape:
+            if not isinstance(shape["Section"], list):
+                sect = [shape["Section"]]
+            else:
+                sect = [x for x in shape["Section"] if x["@N"] == "Property"]
+            # sect is a list or ordered dict. Get the 'Property' section
+            if [x for x in sect if x["@N"] == "Property"]:
+                props = [x for x in sect if x["@N"] == "Property"][0].get("Row")
+            # props is now the 'Property' section of the Shape
+            # Loop the properties
+            # props is ordered_dict based on stencil, but list[ordered_dict] for manual
+            if not isinstance(props, list):
+                props = [props]
+            for prop in props:
+                # get the property row_id
+                row_id = None
+                if str(prop.get("@N")).lower().__contains__("row_"):
+                    row_id = prop.get("@N")
+                # region get the key and value
+                key, value = None, None
+                if "Cell" not in prop:
+                    continue
+                if isinstance(prop.get("Cell"), dict):
+                    # it is a value that belongs to a template label
+                    label = [x for x in labels if x["@N"] == row_id]
+                    kvps = None
+                    if label:
+                        kvps = label[0]["Cell"]
+                    rw_k = [x for x in kvps if x["@N"] == "Label"]
+                    if rw_k:
+                        key = rw_k[0]["@V"]
+                    value = prop.get("Cell")["@V"]
+                else:
+                    rw_k = [x for x in prop.get("Cell") if x["@N"] == "Label"]
+                    if rw_k:
+                        key = rw_k[0]["@V"]
+                    rw_l = [x for x in prop.get("Cell") if x["@N"] == "Value"]
+                    if rw_l:
+                        value = rw_l[0]["@V"]
+                    # endregion
+                if key is not None and value is not None:
+                    properties.update({"row_id": row_id, str(key).lower(): value})
+
+        if "Section" in shape:
+            for rw in shape["Section"]:
+                if str(rw).lower().__contains__("('@v', 'exclusive gateway')") and str(rw).lower().__contains__("('@n', 'type')"):
+                    properties.update({"type": "exclusive gateway"})
+                    properties.update({"name": ""})
+                if str(rw).lower().__contains__("('@v', 'parallel gateway')") and str(rw).lower().__contains__("('@n', 'type')"):
+                    properties.update({"type": "parallel gateway"})
+                    properties.update({"name": ""})
+
+        if "type" not in shape:
+            count = 1
+            while True:
+                if f"visio/pages/page{count}.xml" in self.root:
+                    if not [x for x in self.root[f"visio/pages/page{count}.xml"].get("PageContents").get("Connects").get("Connect") if x["@FromCell"] == "EndX" and x["@ToSheet"] == shape["@ID"]]:
+                        properties.update({"IsStart": True})
+                        properties.update({"name": "Start"})
+                        properties.update({"type": "shape"})
+                    if not [x for x in self.root[f"visio/pages/page{count}.xml"].get("PageContents").get("Connects").get("Connect") if x["@FromCell"] == "BeginX" and x["@ToSheet"] == shape["@ID"]]:
+                        properties.update({"name": "End"})
+                        properties.update({"type": "shape"})
+                    count += 1
+                else:
+                    break
+        if str(properties["name"]).lower().__contains__("gateway"):
+            properties.update({"name": ""})
+        properties.update({"type": str(properties["type"]).lower()})
+        for k, v in properties.items():
+            setattr(retn, k, v)
+        return retn
+
 # Test
 # engine = WorkflowEngine()
-# doc = engine.open(fr"c:\\temp\\test2.xml")  # c:\\temp\\test.xml
+# doc = engine.open(fr"c:\\temp\\test2.vsdx")  # c:\\temp\\test.xml
 # steps = engine.get_flow(doc)
 # engine.run_flow(steps)
